#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <limits.h>
#include "SDL.h"
#include "ui.h"

extern void UpdateGame(unsigned long ulButtons, unsigned long ulJoystick);
extern void RenderScreen(unsigned short *pus);

static unsigned short g_usScreen[320*240];
static SDL_Surface *pScreen = NULL;
static pthread_t thr = 0;
static volatile int thr_exit = 0;
static volatile int g_bUIThreadRunning = 0;

static void *InitScreenSDL()
{
	SDL_Init(SDL_INIT_VIDEO);
	pScreen = SDL_SetVideoMode(320, 240, 32, SDL_SWSURFACE);
	memset(g_usScreen, 0, sizeof(g_usScreen));
	return g_usScreen;
}

static void *FlipScreenSDL()
{
	unsigned long ul, r, g, b;
	unsigned short us;
	unsigned long *ptr = (unsigned long *) pScreen->pixels;
	int x, y;

	for(y = 0; y < 240; y++)
	{
		for(x = 0; x < 320; x++)
		{
			us = g_usScreen[y * 320 + x];
			r = (us >> 11) & 0x1F;
			g = (us >> 5) & 0x3F;
			b = us & 0x1F;
			ul = ((r * 8) << 16) + ((g * 4) << 8) + (b * 8);
			ptr[y * (pScreen->pitch / 4) + x] = ul;
		}
	}
	SDL_UpdateRect(pScreen, 0, 0, 320, 240);
	return g_usScreen;
}

static void CleanupScreenSDL()
{
	SDL_Quit();
}


//#include "splash.c"

//void QuickInitUI()
//{
//  InitScreenSDL();
//    FlipScreenSDL();
//}

static int g_bUp = 0;
static int g_bDown = 0;
static int g_bLeft = 0;
static int g_bRight = 0;
static int g_bL = 0;
static int g_bR = 0;
static int g_bStart = 0;
static int g_bSelect = 0;
static int g_bX = 0;
static int g_bY = 0;
static int g_bA = 0;
static int g_bB = 0;


unsigned long dzzGetButtons()
{
	unsigned long ul = 0;

	SDL_Event event;

	while(SDL_PollEvent(&event))
	{
		switch(event.type)
		{
			case SDL_KEYDOWN:
				if(event.key.keysym.sym == SDLK_LEFT)
					g_bLeft = 1;
				if(event.key.keysym.sym == SDLK_RIGHT)
					g_bRight = 1;
				if(event.key.keysym.sym == SDLK_UP)
					g_bUp = 1;
				if(event.key.keysym.sym == SDLK_DOWN)
					g_bDown = 1;
				if(event.key.keysym.sym == SDLK_TAB)
					g_bL = 1;
				if(event.key.keysym.sym == SDLK_BACKSPACE)
					g_bR = 1;
				if(event.key.keysym.sym == SDLK_SPACE)
					g_bX = 1;
				if(event.key.keysym.sym == SDLK_LALT)
					g_bY = 1;
				if(event.key.keysym.sym == SDLK_LCTRL)
					g_bA = 1;
				if(event.key.keysym.sym == SDLK_LSHIFT)
					g_bB = 1;
				if(event.key.keysym.sym == SDLK_RETURN)
					g_bStart = 1;
				if(event.key.keysym.sym == SDLK_ESCAPE)
					g_bSelect = 1;
				break;
			case SDL_KEYUP:
				if(event.key.keysym.sym == SDLK_LEFT)
					g_bLeft = 0;
				if(event.key.keysym.sym == SDLK_RIGHT)
					g_bRight = 0;
				if(event.key.keysym.sym == SDLK_UP)
					g_bUp = 0;
				if(event.key.keysym.sym == SDLK_DOWN)
					g_bDown = 0;
				if(event.key.keysym.sym == SDLK_TAB)
					g_bL = 0;
				if(event.key.keysym.sym == SDLK_BACKSPACE)
					g_bR = 0;
				if(event.key.keysym.sym == SDLK_SPACE)
					g_bX = 0;
				if(event.key.keysym.sym == SDLK_LALT)
					g_bY = 0;
				if(event.key.keysym.sym == SDLK_LCTRL)
					g_bA = 0;
				if(event.key.keysym.sym == SDLK_LSHIFT)
					g_bB = 0;
				if(event.key.keysym.sym == SDLK_RETURN)
					g_bStart = 0;
				if(event.key.keysym.sym == SDLK_ESCAPE)
					g_bSelect = 0;
				break;
		}
	}
	if(g_bL)
		ul |= GP2X_L;
	if(g_bR)
		ul |= GP2X_R;
	if(g_bStart)
		ul |= GP2X_START;
	if(g_bSelect)
		ul |= GP2X_SELECT;
	if(g_bX)
		ul |= GP2X_X;
	if(g_bY)
		ul |= GP2X_Y;
	if(g_bA)
		ul |= GP2X_A;
	if(g_bB)
		ul |= GP2X_B;
	return ul;
}

int dzzShiftDown()
{
  if(dzzGetButtons() & GP2X_R)
    return 1;
  return 0;
}

unsigned long dzzGetJoystick()
{
	dzzGetButtons();
	if(g_bLeft & g_bUp)
		return JOYDIR_UL;
	if(g_bRight & g_bUp)
		return JOYDIR_UR;
	if(g_bRight & g_bDown)
		return JOYDIR_DR;
	if(g_bLeft & g_bDown)
		return JOYDIR_DL;
	if(g_bLeft)
		return JOYDIR_L;
	if(g_bRight)
		return JOYDIR_R;
	if(g_bUp)
		return JOYDIR_U;
	if(g_bDown)
		return JOYDIR_D;
	return JOYDIR_NONE;
}

#define LONG_LIMIT (LONG_MAX/1000)

unsigned int g_ulFrameTimer = 0;

unsigned long GetTime( void )
{
  struct timeval tv;
  unsigned long time;
  gettimeofday(&tv, NULL);
  time = tv.tv_usec/1000;
  return time;
}

void dzzStartFrameTimer()
{
  g_ulFrameTimer = GetTime();
}

int dzzGetFrameTimer()
{
  unsigned long ul;

  ul = GetTime();
  if(ul > g_ulFrameTimer)
    return (int) (ul - g_ulFrameTimer);
  // wrapped around
  return (int) (ul + (LONG_LIMIT-g_ulFrameTimer));
}

void dzzMicroSleep(int nMicros)
{
	 usleep((unsigned long) nMicros);
}

static void *UIThread(void *none)
{
	unsigned long ulButtons;
	unsigned long ulJoystick;

	InitScreenSDL();
	g_bUIThreadRunning = 1;
	// set up stuff
	
	
	while(!thr_exit)
	{
		// do any game updating.  About all we'd do here is
		// autoscroll the map
		ulButtons = dzzGetButtons();
		ulJoystick = dzzGetJoystick();
		UpdateGame(ulButtons, ulJoystick);

		// draw the screen buffer
		// using the memscreen and current state, etc
		RenderScreen(g_usScreen);
		// wait for vertical sync
		// while(dzzGetFrameTimer() < 16.66)
      		dzzMicroSleep(5000);
		// flip the screen
		FlipScreenSDL();
	}
	CleanupScreenSDL();
	g_bUIThreadRunning = 0;
	return 0;
}

void StartUIThread()
{
	thr=0;
  	thr_exit=0;
	pthread_create(&thr, NULL, UIThread, NULL);
}

void ShutdownUIThread()
{
	thr_exit=1;
	while(g_bUIThreadRunning)
	{
		usleep(10000);
	}
	thr=0;
}

void dzzStartJoystickTimer()
{
}

int dzzGetJoystickTimer()
{
	return 0;
}



void dzzStartStickClickTimer()
{

}

int dzzGetStickClickTimer()
{
return 0;
}
